@@ -300,26 +300,41 @@ class WarningHunter(object):
                               reference)
 
         def _process_function(function, namespace):
+            reference = function.body is None
             if function.return_type:
                 return_type = function.return_type
-                _add_variable(return_type, namespace)
+                _add_variable(return_type, namespace, reference)
 
             for s in function.specializations:
                 _add_variable(s, namespace, not function.body)
 
             templated_types = function.templated_types or ()
             for p in function.parameters:
-                if p.type.name not in templated_types:
-                    if function.body and p.name and p.type.name:
+                node = p.type
+                if node.name not in templated_types:
+                    if function.body and p.name:
                         # Assume that if the the function has a body and a name
                         # the parameter type is really used.
                         # NOTE(nnorwitz): this is over-aggressive. It would be
                         # better to iterate through the body and determine
                         # actual uses based on local vars and data members
                         # used.
-                        _add_use(p.type.name, namespace)
+                        _add_use(node.name, namespace)
+                    elif (
+                        p.default and
+                        p.default[0].name != '0' and
+                        p.default[0].name != 'NULL' and
+                        p.default[0].name != 'nullptr'
+                    ):
+                        _add_use(node.name, namespace)
+                    elif node.reference or node.pointer or reference:
+                        _add_reference(node.name, namespace)
                     else:
-                        _add_variable(p.type, namespace)
+                        _add_use(node.name, namespace)
+                    _add_template_use(node.name,
+                                      node.templated_types,
+                                      namespace,
+                                      reference)
 
         def _process_function_body(function, namespace):
             previous = None
@@ -485,7 +500,7 @@ class WarningHunter(object):
         for name, declared_only in declared_only_symbols.items():
             if declared_only:
                 node = public_symbols[name]
-                if not node.templated_types:
+                if node.templated_types is None:
                     msg = "'{}' declared but not defined".format(name)
                     self._add_warning(msg, node, primary_header.filename)
 
